/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.farm;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;
import nl.cloudfarming.client.farm.model.Farm;
import org.netbeans.core.api.multiview.MultiViews;
import org.netbeans.spi.project.ui.support.CommonProjectActions;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.openide.actions.RenameAction;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 * This is the node you actually see in the project tab for the project
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
final class FarmNode extends FilterNode implements Serializable, Lookup.Provider {

    private final FarmProject project;
    private final Farm farm;
    private boolean userChange = true;

    public FarmNode(Farm farm, Node projectFolderNode, FarmProject project) throws DataObjectNotFoundException {
        super(projectFolderNode,
                //                            new FilterNode.Children(projectFolderNode),
                NodeFactorySupport.createCompositeChildren(project, "Projects/nl-cloudfarming-client-farm/Nodes"),
                //                    //The projects system wants the project in the Node's lookup.
                //                    //NewAction and friends want the original Node's lookup.
                //                    //Make a merge of both
                new ProxyLookup(Lookups.fixed(project),
                project.getLookup(),
                projectFolderNode.getLookup()));
        this.project = project;
        this.farm = farm;
        disableDelegation(DELEGATE_SET_NAME);
        disableDelegation(DELEGATE_GET_NAME);
        setName(farm.getName());
        setDisplayName(getName());

        farm.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (Farm.PROP_NAME.equals(evt.getPropertyName())) {
                    userChange = false;
                    setName((String) evt.getNewValue());
                    setDisplayName(getName());
                    userChange = true;
                }
            }
        });
    }

    @Override
    public void setName(String name) {
        super.setName(name);
        if (userChange) {
            farm.setName(name);
        }
    }

    @Override
    public Action[] getActions(boolean arg0) {
        List<Action> actions = new ArrayList();
        
        actions.add(new OpenFarmDetailsAction());
        actions.add(CommonProjectActions.setAsMainProjectAction());
        actions.add(SystemAction.get(RenameAction.class));
//        actions.add(CommonProjectActions.closeProjectAction());
        
        for (Action action : Utilities.actionsForPath("FarmProjects/Actions")) {
            actions.add(action);
        }

        actions.add(new RemoveFarmAction(project));

        return actions.toArray(new Action[actions.size()]);
    }

    @Override
    public Image getIcon(int type) {
        return ImageUtilities.loadImage(FarmProject.ICON);
    }

    @Override
    public Image getOpenedIcon(int type) {
        return getIcon(type);
    }

    @Override
    public String getDisplayName() {
        return farm.getName();
    }

    @Override
    public boolean canCopy() {
        return false;
    }

    @Override
    public boolean canCut() {
        return false;
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @NbBundle.Messages(value = "Farm open details action name=Open")
    private final class OpenFarmDetailsAction extends AbstractAction {

        public OpenFarmDetailsAction() {
            super(Bundle.Farm_open_details_action_name());
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            TopComponent tc = MultiViews.createMultiView("application/x-agrosense-farm", FarmNode.this);
            tc.open();
            tc.requestActive();
        }
    }

    private final class RemoveFarmAction extends AbstractAction {

        private FarmProject project;
        private static final String TITLE = "Delete";

        public RemoveFarmAction(FarmProject project) {
            putValue(NAME, TITLE);
            this.project = project;
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            // TODO: Would this be a correct way to remove a Farm?

            int option = JOptionPane.showConfirmDialog(
                    null,
                    "Are you sure you want to delete the farm " + project.getFarm().getName() + "?",
                    "Warning!",
                    JOptionPane.YES_NO_OPTION);

            if (option == JOptionPane.YES_OPTION) {
                CommonProjectActions.closeProjectAction().actionPerformed(null);
                try {
                    //TODO: unregister savables for this project
                    project.getProjectDirectory().delete();
                    // remove from managed farms:
                    FarmProjectRegistry.getInstance().remove(project);
                } catch (IOException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }

        }
    }
}
